"""Views module"""

from django.contrib.auth.mixins import LoginRequiredMixin
from django.http.response import HttpResponseBadRequest, HttpResponseRedirect
from django.views.generic.base import View

from discourse_django_sso import utils
from discourse_django_sso.utils import ConsumerType


class SSOProviderView(LoginRequiredMixin, View):
    """
    View that implements sso.
    """

    sso_secret = None
    sso_redirect = None
    consumer_type = ConsumerType.DISCOURSE

    def get(self, request, **kwargs):  # pylint: disable=unused-argument
        """Performs the SSO"""
        try:
            sso = request.GET['sso']
            sig = request.GET['sig']
        except KeyError:
            return HttpResponseBadRequest()
        redirect = utils.SSOProviderService(
            sso_key=self.sso_secret
        ).get_signed_url(
            user=request.user,
            redirect_to=self.sso_redirect,
            sso=sso,
            signature=sig,
            consumer_type=self.consumer_type
        )
        if redirect is None:
            return HttpResponseBadRequest()
        return HttpResponseRedirect(
            redirect_to=redirect
        )





# coding=utf-8
"""Test SSO producer utils."""
import base64

import pytest

from discourse_django_sso.utils import SSOProducerUtils, ConsumerType

# pylint: disable=missing-docstring,redefined-outer-name


@pytest.fixture()
def sso_producer(
        example_user,
        example_secret,
        example_producer_url,
        example_sig,
        example_sso
):
    return SSOProducerUtils(
        sso_key=example_secret,
        consumer_url=example_producer_url,
        user=example_user,
        sso=example_sso,
        sig=example_sig
    )


def test_verify_signature(sso_producer):
    assert sso_producer.verify_signature()


def test_get_noonce(sso_producer):
    assert sso_producer.get_nonce() == b'cb68251eefb5211e58c00ff1395f0c0b'


def test_get_response_params(sso_producer, example_user):
    assert sso_producer.get_response_params() == (
        ('nonce', b'cb68251eefb5211e58c00ff1395f0c0b'),
        ('email', example_user.email),
        ('username', example_user.username),
        ('external_id', example_user.id),
    )


def test_get_signed_payload(sso_producer):
    assert sso_producer.get_signed_payload() == {
        'sig': b'30cadd6c051e07900ffc27323354f90d03abcee835b55f458de33a51d5dd3341',
        'sso': b'bm9uY2U9Y2I2ODI1MWVlZmI1MjExZTU4YzAwZmYxMzk1ZjBjMGImZW1haWw9Zm9vJTQw'
               b'YmFyLnBsJnVzZXJuYW1lPWZvbyZleHRlcm5hbF9pZD0x'
    }


def test_get_sso_redirect(sso_producer):
    expected = 'http://example.org/sso?sig=foo&sso=bar'
    actual = sso_producer.get_sso_redirect((
        ('sig', 'foo'),
        ('sso', 'bar'),
    ))
    assert expected == actual


def test_validate_eol(sso_producer):
    """
    Discourse produces SSO that ends with a new line
    it you accidentally trim it you'll get invalid key,
    and will not have a good time debugging it.
    """
    sso_producer.sso = b"bm9uY2U9Y2I2ODI1MWVlZmI1MjExZTU4YzAwZmYxMzk1ZjBjMGI="
    with pytest.raises(ValueError):
        sso_producer.validate()
    sso_producer.validate(consumer_type=ConsumerType.OTHER)


def test_validate_noonce(sso_producer):
    sso_producer.sso = base64.b64encode(b"something_else=231")
    with pytest.raises(ValueError):
        sso_producer.validate(consumer_type=ConsumerType.OTHER)





# coding=utf-8

"""Tests for base SSO generator."""

import base64
from urllib.parse import parse_qs, unquote

import pytest

from discourse_django_sso import utils

# pylint: disable=missing-docstring
# pylint: disable=protected-access,redefined-outer-name


@pytest.fixture()
def base_generator(example_secret):
    return utils.BaseSSOGenerator(example_secret)


def test_signature(base_generator, example_sso, example_sig):
    assert base_generator._get_payload_singature(example_sso) == example_sig


def test_verify_payload(base_generator, example_sso, example_sig):
    assert base_generator._verify_payload(example_sso, example_sig)
    assert base_generator._verify_payload(example_sso.decode('ascii'), example_sig)
    assert base_generator._verify_payload(example_sso, example_sig.decode('ascii'))
    assert base_generator._verify_payload(example_sso.decode('ascii'), example_sig.decode('ascii'))


def test_verify_payload_negative(base_generator, example_sso, example_sig):
    assert not base_generator._verify_payload(example_sso, '')
    assert not base_generator._verify_payload(example_sso + b'abc', example_sig)


def test_prepare_signed_payload(base_generator):
    nonce_dict = {
        b'nonce': b'cb68251eefb5211e58c00ff1395f0c0b'
    }

    payload = base_generator.prepare_signed_payload(nonce_dict)
    assert payload == {
        'sig': b'1ce1494f94484b6f6a092be9b15ccc1cdafb1f8460a3838fbb0e0883c4390471',
        'sso': b'bm9uY2U9Y2I2ODI1MWVlZmI1MjExZTU4YzAwZmYxMzk1ZjBjMGI='
    }

    # We check that sso parses to expected nonce.
    assert parse_qs(base64.b64decode(payload['sso'])) == {
        b'nonce': [b'cb68251eefb5211e58c00ff1395f0c0b']
    }


@pytest.mark.django_db
def test_sso_provider(example_secret, example_producer_url, example_user):

    expected_url = (
        'http://example.org/sso?sso=bm9uY2U9ZWFmMmQ3NDY'
        '3NDk3ZjhiYTM1OGU0NTRhNzkxMWYxNmQmZW1haWw9Zm9vJ'
        'TQwYmFyLnBsJnVzZXJuYW1lPWZvbyZleHRlcm5hbF9pZD0'
        'x&sig=cb690d472636902abd5e43a88db8d502d04dad22'
        '4516d30a90efa74ba4a2ae00'
    )

    service = utils.SSOProviderService(
        sso_key=example_secret,
    )

    response = service.get_signed_url(
        redirect_to=example_producer_url,
        user=example_user,
        sso=unquote(
            'bm9uY2U9ZWFmMmQ3NDY3NDk3ZjhiYTM1OGU0NTRhNzkxMWYxNmQmcmV0dXJ'
            'u%0AX3Nzb191cmw9aHR0cCUzQSUyRiUyRmxvY2FsaG9zdCUzQTgwODElMkZ'
            'zZXNz%0AaW9uJTJGc3NvX2xvZ2lu%0A'
        ),
        signature='cd3181ea14dde017a8980c455c19fd10f679fb1fbd778be7125c7602bb792b12'
    )

    assert response == expected_url



# coding=utf-8

"""Test sso view."""

import base64
import hashlib
import hmac
from urllib.parse import urlencode

import pytest
from django.test.client import Client
from django.urls.base import reverse
from django.utils.encoding import force_bytes

from discourse_django_sso_test_project import settings


# pylint: disable=missing-docstring,redefined-outer-name


@pytest.fixture()
def noonce():
    return "random_noonce"


@pytest.fixture()
def sso_key():
    return force_bytes(settings.DISCOURSE_SSO_KEY)


@pytest.fixture()
def sso_contents(noonce, sso_key):
    sso = base64.b64encode(urlencode({'nonce': noonce}).encode('ascii')) + b'\n'
    sig = force_bytes(hmac.new(sso_key, force_bytes(sso), digestmod=hashlib.sha256).hexdigest())
    return {
        "sig": sig,
        "sso": sso
    }


def test_view(admin_client: Client, sso_contents):
    response = admin_client.get(reverse('sso'), data=sso_contents)
    assert response.status_code == 302
    expected_response = (
        "http://localhost/test?sso=bm9uY2U9cmFuZG9tX25vb25jZ"
        "SZlbWFpbD1hZG1pbiU0MGV4YW1wbGUuY29tJnVzZXJuYW1lPWFk"
        "bWluJmV4dGVybmFsX2lkPTE%3D&sig=f79d43aba7fe196c1f3b"
        "533ca9f49cabdf09d49dccda7376524a759b364f56ee"
    )
    assert response.url == expected_response


def test_view_empty_get(admin_client: Client):
    response = admin_client.get(reverse('sso'), data={})
    assert response.status_code == 400


def test_invalid_data(admin_client: Client):
    response = admin_client.get(reverse('sso'), data={
        "sso": "bm9uY2U9Y2I2ODI1MWVlZmI1MjExZTU4YzAwZmYxMzk1ZjBjMGI=\n",
        "sig": "invalid_sig"
    })
    assert response.status_code == 400


def test_view_no_newline(admin_client: Client):
    response = admin_client.get(reverse('sso'), data={
        "sso": "bm9uY2U9Y2I2ODI1MWVlZmI1MjExZTU4YzAwZmYxMzk1ZjBjMGI=",
        "sig": "invalid_sig"
    })
    assert response.status_code == 400

